@@ -8,6 +8,7 @@ from grid.cli.grid_history import history
 from grid.cli.grid_interactive import interactive
 from grid.cli.grid_login import login
 from grid.cli.grid_logs import logs
+from grid.cli.grid_metrics import metrics
 # SLURM
 from grid.cli.grid_slurm import slurm
 from grid.cli.grid_ssh_keys import ssh_keys
@@ -24,6 +25,7 @@ __all__ = [
     'credentials',
     'history',
     'logs',
+    'metrics',
     'interactive',
     'artifacts',
     'deletecluster',
