@@ -1,24 +1,9 @@
 from typing import List, Optional
 
 import click
-from click.exceptions import BadArgumentUsage
 
 from grid import Grid
-from grid.utilities import is_experiment
-
-
-def _check_is_experiment(_ctx, _param, value):
-    """
-    Click callback that checks if passed list of experiments are
-    actual experiments.
-    """
-    for experiment in value:
-        if not is_experiment(experiment):
-            raise BadArgumentUsage(
-                f'{experiment} is not an experiment.'
-                'You can only download artifacts for experiments.')
-
-    return value
+from grid.utilities import check_is_experiment
 
 
 @click.command()
@@ -31,7 +16,7 @@ def _check_is_experiment(_ctx, _param, value):
                 type=str,
                 required=True,
                 nargs=-1,
-                callback=_check_is_experiment)
+                callback=check_is_experiment)
 def artifacts(experiments: List[str],
               download_dir: Optional[str] = None) -> None:
     """Downloads artifacts for a given experiment or set of experiments."""
