@@ -17,6 +17,7 @@ import grid.globals as env
 from grid.tracking import Segment
 from grid.tracking import TrackingEvents
 from grid.types import ObservableType
+from grid.utilities import check_description_isnt_too_long
 
 
 def _print_submission_message(instance_type: str, provider: str,
@@ -172,6 +173,13 @@ def interactive(ctx) -> None:
               required=False,
               help='Name for this run',
               callback=_check_run_name_is_valid)
+@click.option('--grid_description',
+              '--g_description',
+              'description',
+              type=str,
+              required=False,
+              help='Interactive node description; useful for note-keeping',
+              callback=check_description_isnt_too_long)
 @click.option('--grid_credential',
               '--g_credential',
               'credential',
@@ -182,7 +190,7 @@ def interactive(ctx) -> None:
               '--g_instance_type',
               'instance_type',
               type=str,
-              default='t2.xlarge',
+              default='t2.medium',
               help='Instance type to start training session in')
 @click.option('--grid_disk_size',
               '--g_disk_size',
@@ -215,8 +223,9 @@ def interactive(ctx) -> None:
     'Absolute path to mount Datastore in interactive node. (default to ~/datastore)'
 )
 def create(config: Optional[Dict], name: Optional[str],
-           credential: Optional[str], instance_type: str,
-           datastore_name: Optional[str], datastore_version: Optional[str],
+           description: Optional[str], credential: Optional[str],
+           instance_type: str, datastore_name: Optional[str],
+           datastore_version: Optional[str],
            datastore_mount_dir: Optional[str],
            disk_size: Optional[int]) -> None:
     """Creates an interactive node."""
@@ -253,7 +262,9 @@ def create(config: Optional[Dict], name: Optional[str],
     _grid_config_yaml = yaml.dump(_grid_config)
 
     #  Send to client.
-    client.create_interactive_node(config=_grid_config_yaml, name=name)
+    client.create_interactive_node(config=_grid_config_yaml,
+                                   name=name,
+                                   description=description)
 
     #  Shows a friendly submission message to the
     #  user.
@@ -267,6 +278,22 @@ def create(config: Optional[Dict], name: Optional[str],
         name=name)
 
 
+@interactive.command()
+@click.argument('interactive_node', type=str, nargs=1)
+def pause(interactive_node: str) -> None:
+    """Pauses an interactive node."""
+    client = Grid()
+    client.pause_interactive_node(interactive_node_name=interactive_node)
+
+
+@interactive.command()
+@click.argument('interactive_node', type=str, nargs=1)
+def resume(interactive_node: str) -> None:
+    """Resumes an interactive node."""
+    client = Grid()
+    client.resume_interactive_node(interactive_node_name=interactive_node)
+
+
 @interactive.command()
 @click.argument('interactive_node', type=str, nargs=1)
 def delete(interactive_node: str) -> None:
