@@ -14,6 +14,7 @@ import grid.globals as env
 from grid.tracking import Segment
 from grid.tracking import TrackingEvents
 from grid.types import WorkflowType
+from grid.utilities import check_description_isnt_too_long
 from grid.utilities import is_experiment
 
 
@@ -102,14 +103,6 @@ def _check_run_name_is_valid(_ctx, _param, value):
     return value
 
 
-def _check_description_isnt_too_long(ctx, _param, value):
-    """Click callback that checks if the description isn't too long."""
-    if value is not None:
-        if len(value) > 200:
-            raise click.BadParameter('Description should have at most '
-                                     f'200 characters. Yous has {len(value)}.')
-
-
 def _check_ignore_warnings_flag(ctx, _param, value):
     """
     Click callback that assigns the value of the ignore warnigns callback
@@ -160,7 +153,8 @@ def _generate_default_grid_config(client: Grid,
                                   processes: Optional[int] = None,
                                   datastore_name: Optional[str] = None,
                                   datastore_version: Optional[int] = None,
-                                  datastore_mount_dir: Optional[str] = None):
+                                  datastore_mount_dir: Optional[str] = None,
+                                  framework: Optional[str] = None):
     """
     Generates a new default config file for user if user hasn't
     submitted one.
@@ -181,6 +175,7 @@ def _generate_default_grid_config(client: Grid,
                 'region': defaults['region']
             },
             'train': {
+                'framework': framework,
                 'datastore_name': datastore_name,
                 'datastore_version': datastore_version,
                 'datastore_mount_dir': datastore_mount_dir,
@@ -237,6 +232,32 @@ def _check_is_valid_extension(ctx, _param, value):
         return value
 
 
+def _get_instance_types(ctx, args, incomplete):
+    # TODO: these should be retrieved from backend
+    instance_types = [
+        "p3.16xlarge",
+        "p3dn.24xlarge",
+        "p3.16xlarge",
+        "g4dn.metal",
+        "p2.8xlarge",
+        "p3.8xlarge",
+        "g4dn.12xlarge",
+        "g3.16xlarge",
+        "g3.8xlarge",
+        "p3.2xlarge",
+        "g4dn.16xlarge",
+        "g4dn.8xlarge",
+        "g4dn.4xlarge",
+        "g4dn.2xlarge",
+        "p2.xlarge",
+        "g3.4xlarge",
+        "g3s.xlarge",
+        "t2.large",
+        "t2.medium",
+    ]
+    return [x for x in instance_types if incomplete in x]
+
+
 @click.command(context_settings=dict(ignore_unknown_options=True))
 @click.option('--grid_config',
               '--g_config',
@@ -258,7 +279,7 @@ def _check_is_valid_extension(ctx, _param, value):
               type=str,
               required=False,
               help='Run description; useful for note-keeping',
-              callback=_check_description_isnt_too_long)
+              callback=check_description_isnt_too_long)
 @click.option('--grid_credential',
               '--g_credential',
               'credential',
@@ -283,7 +304,8 @@ def _check_is_valid_extension(ctx, _param, value):
               'instance_type',
               type=str,
               default='t2.xlarge',
-              help='Instance type to start training session in')
+              help='Instance type to start training session in',
+              autocompletion=_get_instance_types)
 @click.option('--grid_gpus',
               '--g_gpus',
               'gpus',
@@ -349,6 +371,13 @@ def _check_is_valid_extension(ctx, _param, value):
               type=str,
               required=False,
               help='Directory to mount Datastore in training job')
+@click.option('--grid_framework',
+              '--g_framework',
+              'framework',
+              type=str,
+              required=False,
+              default='lightning',
+              help='Framework to use in training')
 @click.option('--ignore_warnings',
               is_flag=True,
               required=False,
@@ -367,8 +396,9 @@ def train(ctx, script: str, entrypoint: str, config: Optional[Dict],
           ignore_warnings: bool, script_args: Optional[Any],
           memory: Optional[str], cpus: Optional[int],
           datastore_name: Optional[str], datastore_version: Optional[str],
-          datastore_mount_dir: Optional[str]) -> None:
-    """Trains scripts and blueprints"""
+          datastore_mount_dir: Optional[str],
+          framework: Optional[str]) -> None:
+    """Trains scripts in remote infrastructure"""
 
     #  Captures all other user arguments that are not
     #  parsed automatically.
@@ -410,7 +440,8 @@ def train(ctx, script: str, entrypoint: str, config: Optional[Dict],
             cpus=cpus,
             datastore_name=datastore_name,
             datastore_version=datastore_version,
-            datastore_mount_dir=datastore_mount_dir)
+            datastore_mount_dir=datastore_mount_dir,
+            framework=framework)
 
     else:
         _grid_config = config
