@@ -1,4 +1,5 @@
 # Copyright 2020 Grid AI Inc.
+from array import array
 import ast
 import base64
 import csv
@@ -9,6 +10,8 @@ from pathlib import Path
 import time
 from typing import Any, Callable, Dict, List, Optional
 from urllib.parse import urljoin
+from urllib.parse import urlsplit
+import zlib
 
 import click
 from dateutil.parser import parse as date_string_parse
@@ -31,6 +34,7 @@ from grid.commands import CredentialsMixin
 from grid.commands import DependencyMixin
 from grid.commands import WorkflowChecksMixin
 from grid.commands.git import execute_git_command
+from grid.datastore import create_datastore_session
 from grid.datastore import DatastoreUploadSession
 from grid.downloader import DownloadableObject
 from grid.downloader import Downloader
@@ -921,7 +925,8 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
         return success
 
-    def create_interactive_node(self, config: str, name: str) -> bool:
+    def create_interactive_node(self, config: str, name: str,
+                                description: str) -> bool:
         """
         Creates an interactive node via Grid.
 
@@ -931,18 +936,14 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
             String representation of YAML file
         name: str
             Name of interactive node to use
+        description: str
+            Description of interactive node
 
         Returns
         -------
         success: bool
             Truthy if operation is successful.
         """
-        # Check user Github token for user.
-        self._check_user_github_token()
-
-        # Check if active directory is a Github repository.
-        self._check_github_repository()
-
         spinner = yaspin.yaspin(text="Creating Interactive node ...",
                                 color="yellow")
         spinner.start()
@@ -952,32 +953,16 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
         config_encoded = base64.b64encode(
             config.encode('ascii')).decode('ascii')
 
-        #  Get commit SHA
-        commit_sha = execute_git_command(['rev-parse', 'HEAD'])
-
-        #  Get repo name
-        github_repository = execute_git_command(
-            ["config", "--get", "remote.origin.url"])
-
-        #  Clean up the repo name
-        github_repository = github_repository.replace('git@github.com:', '')
-        github_repository = github_repository.replace('.git', '')
-        github_repository = github_repository.replace('https://', '')
-        github_repository = github_repository.replace('http://', '')
-        github_repository = github_repository.replace('github.com/', '')
-
         #  Cancel the entire Run otherwise.
         mutation = gql("""
         mutation (
             $name: ID!
+            $description: String
             $configString: String!
-            $githubRepository: ID!
-            $commitSha: String
         ) {
             createInteractiveNode(properties: {
-                                    name: $name, configString: $configString,
-                                    githubRepository: $githubRepository,
-                                    commitSha: $commitSha
+                                    name: $name, description: $description,
+                                    configString: $configString
                                   }) {
                 success
                 message
@@ -987,11 +972,9 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
         params = {
             'name': name,
+            'description': description,
             'configString': config_encoded,
-            'githubRepository': github_repository,
-            'commitSha': commit_sha
         }
-        success = False
         try:
             result = self.client.execute(mutation, variable_values=params)
         except Exception as e:  # skipcq: PYL-W0703
@@ -1014,6 +997,106 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
         return success
 
+    def pause_interactive_node(self, interactive_node_name: str) -> None:
+        """
+        Pauses an interactive node.
+
+        Parameters
+        ----------
+        interactive_node_name: str
+            Interactive node ID
+        """
+        spinner = yaspin.yaspin(text="Pausing Interactive node ...",
+                                color="yellow")
+        spinner.start()
+
+        mutation = gql("""
+        mutation (
+            $interactiveNodeName: ID!
+        ) {
+            pauseInteractiveNode(interactiveNodeName: $interactiveNodeName) {
+                success
+                message
+            }
+        }
+        """)
+
+        params = {'interactiveNodeName': interactive_node_name}
+
+        success = False
+        try:
+            result = self.client.execute(mutation, variable_values=params)
+        except Exception as e:  # skipcq: PYL-W0703
+            message = ast.literal_eval(str(e))['message']
+            spinner.fail("✘")
+            raise click.ClickException(message)
+
+        success = result['pauseInteractiveNode']['success']
+        if success:
+            spinner.ok("✔")
+            click.echo(f'Interactive node {interactive_node_name} has ' +
+                       'been paused successfully.')
+
+        elif not success:
+            spinner.fail("✘")
+            if env.DEBUG:
+                click.echo(f"→ {result['pauseInteractiveNode']['message']}")
+
+            raise click.ClickException(
+                f"Failed to pause interactive node '{interactive_node_name}'")
+
+        return success
+
+    def resume_interactive_node(self, interactive_node_name: str) -> None:
+        """
+        Resumes an interactive node.
+
+        Parameters
+        ----------
+        interactive_node_name: str
+            Interactive node ID
+        """
+        spinner = yaspin.yaspin(text="Resuming Interactive node ...",
+                                color="yellow")
+        spinner.start()
+
+        mutation = gql("""
+        mutation (
+            $interactiveNodeName: ID!
+        ) {
+            resumeInteractiveNode(interactiveNodeName: $interactiveNodeName) {
+                success
+                message
+            }
+        }
+        """)
+
+        params = {'interactiveNodeName': interactive_node_name}
+
+        success = False
+        try:
+            result = self.client.execute(mutation, variable_values=params)
+        except Exception as e:  # skipcq: PYL-W0703
+            message = ast.literal_eval(str(e))['message']
+            spinner.fail("✘")
+            raise click.ClickException(message)
+
+        success = result['resumeInteractiveNode']['success']
+        if success:
+            spinner.ok("✔")
+            click.echo(f'Interactive node {interactive_node_name} has ' +
+                       'been resumed successfully.')
+
+        elif not success:
+            spinner.fail("✘")
+            if env.DEBUG:
+                click.echo(f"→ {result['resumeInteractiveNode']['message']}")
+
+            raise click.ClickException(
+                f"Failed to resume interactive node '{interactive_node_name}'")
+
+        return success
+
     def delete_interactive_node(self, interactive_node_name: str) -> None:
         """
         Deletes an interactive node from cluster.
@@ -1027,7 +1110,6 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
                                 color="yellow")
         spinner.start()
 
-        #  Cancel the entire Run otherwise.
         mutation = gql("""
         mutation (
             $interactiveNodeName: ID!
@@ -1086,6 +1168,7 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
         ) {
             getExperimentDetails(experimentId: $experimentId) {
+                experimentId
                 status
             }
         }
@@ -1223,7 +1306,7 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
 
     Would you like to proceed? """
-                click.confirm(too_many_lines_message, abort=True)
+                click.confirm(too_many_lines_message, abort=True, default=True)
 
             # Style the log lines.
             styled_logs = []
@@ -1237,7 +1320,7 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
                 else:
                     timestamp = click.style(log['timestamp'], fg='green')
 
-                styled_logs.append(f"[{timestamp}] {log['message']}\n")
+                styled_logs.append(f"[{timestamp}] {log['message']}")
 
             # Either print the logs in the terminal or use the pager
             # to scroll through the logs.
@@ -1266,7 +1349,12 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
             }
             """)
 
-            params = {'experimentId': experiment_id, 'nLines': n_lines}
+            params = {
+                'experimentId':
+                experiment_details['getExperimentDetails']['experimentId'],
+                'nLines':
+                n_lines
+            }
 
             # Create websocket connection.
             try:
@@ -1320,6 +1408,125 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
         #  disappears.
         spinner.stop()
 
+    def experiment_metrics(self,
+                           experiment_id: str,
+                           metric: str,
+                           n_lines: int = 50) -> Dict[str, str]:
+        """
+        Gets experiment scalar logs from a single experiment.
+
+        Parameters
+        ----------
+        experiment_id: str
+            Experiment ID for a single experiment
+        metric: str
+            Metric names
+        n_lines: int, default 50
+            Max number of lines to return
+        """
+        #  Starts spinner.
+        spinner = yaspin.yaspin(text="Fetching metrics ...", color="yellow")
+        spinner.start()
+
+        experiment_details = self.experiment_details(
+            experiment_id=experiment_id)
+
+        # Check if Experiment is queued.
+        state = experiment_details['getExperimentDetails']['status']
+        if state == 'queued':
+            spinner.ok("✔")
+            styled_queued = click.style('queued', fg='yellow')
+            click.echo(f"""
+    Your Experiment is {styled_queued}. Metrics will be available
+    when your Experiment starts.
+            """)
+            spinner.stop()
+            return
+
+        query = gql("""
+        query GetScalarLogs ($experimentId: ID!, $metric: String!, $startIndex: Int, $format: String) {
+            getExperimentScalarLogs(experimentId: $experimentId,
+                                    metric: $metric,
+                                    startIndex: $startIndex,
+                                    format: $format) {
+                values
+                steps
+                nextIndex
+            }
+        }
+        """)
+
+        params = {
+            'experimentId': experiment_id,
+            'metric': metric,
+            'startIndex': 0,
+            'format': 'json'
+        }
+
+        try:
+            result = self.client.execute(query, variable_values=params)
+        #  Raise any other errors that the backend may raise.
+        except Exception as e:  # skipcq: PYL-W0703
+            spinner.fail("✘")
+            if 'Server error:' in str(e):
+                e = str(e).replace('Server error: ', '')[1:-1]
+
+            message = ast.literal_eval(str(e))['message']
+            raise click.ClickException(message)
+
+        metric = result['getExperimentScalarLogs']
+        # The backend will return an empty object if no metrics
+        # are available.
+        if not metric or not metric['steps']:
+            spinner.stop()
+            raise click.ClickException(
+                f'No metrics available for experiment {experiment_id}')
+
+        spinner.ok("✔")
+        spinner.stop()
+
+        # TODO: Change when encoding and compression is fixed in
+        # backend API
+        # def decode(x: bytes, dtype: str):
+        #     buf = zlib.decompress(base64.b64decode(x.encode()))
+        #     out = array(dtype)
+        #     out.frombytes(buf)
+        #     return out
+
+        try:
+            # steps = decode(metric['steps'], 'q')
+            # values = decode(metric['values'], 'd')
+
+            steps = json.loads(metric['steps'])
+            values = json.loads(metric['values'])
+        #  Raise any other errors that the backend may raise.
+        except Exception as e:
+            spinner.fail("✘")
+            if 'Server error:' in str(e):
+                e = str(e).replace('Server error: ', '')[1:-1]
+
+            message = ast.literal_eval(str(e))['message']
+            raise click.ClickException(message)
+
+        if n_lines > 0:
+            steps = steps[-n_lines:]
+            values = values[-n_lines:]
+
+        # Get all metrics in lines
+        lines = [
+            f"{step:>16}  {value:.10f}\n"
+            for step, value in zip(steps, values)
+        ]
+
+        # Print the metrics in the terminal
+        for line in lines:
+            click.echo(line, nl=False)
+
+        #  Makes sure to close the spinner in all situations.
+        #  If we don't do this, the tracker character in the terminal
+        #  disappears.
+        spinner.stop()
+
     def download_experiment_artifacts(self, experiment_id: str,
                                       download_dir: str) -> None:
         """
@@ -1559,27 +1766,12 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
                 f"No session named {session_name} found, please specify an " +
                 "available session")
 
-        spinner = yaspin.yaspin(
-            text=f'Resuming uploading datastore {session.name} ' +
-            f'v{session.version}',
-            color="yellow")
-        spinner.start()
-
-        def update_progress(text: str):
-            spinner.text = text
-
-        session.configure(progress_callback=update_progress,
-                          client=self.client)
+        session.configure(client=self.client)
         try:
             session.upload()
-            spinner.text = "Finished uploading datastore."
-            spinner.ok("✔")
         except Exception as e:  #skipcq: PYL-W0703
-            spinner.fail("✘")
             raise click.ClickException(
                 f'Failed to upload datastore {session.name}. {e}')
-        finally:
-            spinner.stop()
 
     @staticmethod
     def __print_datastore_sessions(sessions: List[DatastoreUploadSession]):
@@ -1622,6 +1814,7 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
                     version
                     size
                     createdAt
+                    snapshotStatus
                 }
             }
         """)
@@ -1630,30 +1823,46 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
         except TransportQueryError as e:
             raise click.ClickException(f"Unable to list datastores: {e}")
 
-        table_cols = ["Credential Id", 'Name', 'Version', 'Size', 'Created']
+        table_cols = [
+            "Credential Id", 'Name', 'Version', 'Size', 'Created', 'Status'
+        ]
         table = BaseObservable.create_table(columns=table_cols)
         for row in result["getDatastores"]:
             created_at = date_string_parse(row["createdAt"])
             created_at = f'{created_at:%Y-%m-%d %H:%M}'
-            size = humanize.naturalsize(row["size"] * (1024**2))
+            size = row["size"]
+            if size:
+                size = humanize.naturalsize(size * (1024**2))
+
+            if row['snapshotStatus'] in {
+                    'pending', 'preparing', 'queued', 'running'
+            }:
+                status = 'Optimizing'
+            else:
+                status = row['snapshotStatus'].title(
+                ) if row['snapshotStatus'] else "Unknown"
             table.add_row(row["credentialId"], row["name"], row["version"],
-                          size, created_at)
+                          size, created_at, status)
 
         console = Console()
         console.print(table)
 
-    def upload_datastore(self, source_dir: str, name: str, credential_id: str):
+    def upload_datastore(self, source: str, name: str, credential_id: str,
+                         compression: bool):
         """
         Uploads datastore to storage
 
         Parameters
         ----------
-        source_dir: str
-           Source local directory to upload from
+        source: str
+           Source to create datastore from, either a local directory or a
+           http url.
         name: str
            Name of datastore
         credential_id: str
             Grid credential id
+        compression: str
+            Enable compression, which is disabled by default
 
         Returns
         -------
@@ -1661,30 +1870,16 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
             Truthy when a datastore has been uploaded
             correctly.
         """
-        spinner = yaspin.yaspin(text=f'Uploading datastore {name}',
-                                color="yellow")
-        spinner.start()
-
-        def update_progress(text: str):
-            spinner.text = text
-
         try:
-            session = DatastoreUploadSession(name=name,
-                                             source_dir=source_dir,
-                                             credential_id=credential_id)
-            session.configure(progress_callback=update_progress,
-                              client=self.client)
+            session = create_datastore_session(name=name,
+                                               source=source,
+                                               credential_id=credential_id,
+                                               compression=compression,
+                                               client=self.client)
             session.upload()
-
-            spinner.text = "Finished uploading datastore."
-            spinner.ok("✔")
-
         except Exception as e:  #skipcq: PYL-W0703
-            spinner.fail("✘")
             raise click.ClickException(
                 f'Failed to upload datastore {name}. {e}')
-        finally:
-            spinner.stop()
 
         return True
 
@@ -1896,36 +2091,36 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
 
         return result['getSlurmStatus']
 
-    def add_ssh_public_key(self, key: str):
+    def add_ssh_public_key(self, key: str, name: str):
         return self.execute_gql("""
             mutation (
                 $publicKey: String!
+                $name: String!
             ) {
-              addSSHPublicKey(publicKey: $publicKey) {
+            addSSHPublicKey(name: $name, publicKey: $publicKey) {
                 message
                 success
                 id
               }
             }
         """,
-                                publicKey=key)
+                                publicKey=key,
+                                name=name)
 
-    def list_public_ssh_keys(self, after: int,
-                             limit: int) -> List[Dict[str, str]]:
+    def list_public_ssh_keys(self, limit: int) -> List[Dict[str, str]]:
         result = self.execute_gql(
             """
             query (
                 $limit: Int!,
-                $after: Int!,
             ) {
-                getPublicSSHKeys(limit: $limit, after:$after) {
+                getPublicSSHKeys(limit: $limit) {
                     id
-                    publicKey
+                    publicKey,
+                    name
               }
             }
         """,
             limit=limit,
-            after=after,
         )
         return result['getPublicSSHKeys']
 
@@ -1947,8 +2142,7 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
         query {
             getInteractiveNodes {
                 name: interactiveNodeName
-                port: sshPort
-                hostname
+                ssh_url: sshUrl
             }
         }
         """)['getInteractiveNodes']
@@ -1965,11 +2159,14 @@ class Grid(CredentialsMixin, WorkflowChecksMixin, DependencyMixin):
         sol = []
         managed_part = [start_marker]
         for node in interactive_nodes:
+            ssh_url = urlsplit(node['ssh_url'])
             managed_part.extend([
                 f"Host {node['name']}",
-                "    User jovyan",
-                f"    Hostname {node['hostname']}",
-                f"    Port {node['port']}",
+                f"    User {ssh_url.username}",
+                f"    Hostname {ssh_url.hostname}",
+                f"    Port {ssh_url.port}",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
             ])
         managed_part.append(end_marker)
