@@ -1,15 +1,17 @@
 from __future__ import annotations
 
 import abc
-from dataclasses import dataclass
 from enum import Enum
 import functools
+import itertools
 import json
 import os
 from pathlib import Path
+import re
 import subprocess
 import sys
 from typing import Collection, Dict, List, Optional, Union
+from urllib.parse import urlparse
 
 from packaging.specifiers import SpecifierSet
 from pipreqs import pipreqs
@@ -19,9 +21,20 @@ import yaml
 
 # https://www.python.org/dev/peps/pep-0610/
 PACKAGE_URL_METAFILE = "direct_url.json"
+ALLOWED_SCHEMES = {
+    'http', 'https', 'file', 'ftp', 'ssh', 'git', 'sftp', 'git+http',
+    'git+https'
+}
 
 
-def execute_conda_command(args: List[str]) -> str:
+class NonComparableError(Exception):
+    """
+    Two Packages are non-comparable
+    """
+    pass
+
+
+def execute_conda_command(args: List[str]):
     process = subprocess.run(["conda"] + args,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
@@ -31,21 +44,24 @@ def execute_conda_command(args: List[str]) -> str:
     return output
 
 
+def get_url_scheme(url: str) -> str:
+    """Parse the URL and get the scheme"""
+    return urlparse(url).scheme
+
+
+def is_valid_url(url: str) -> bool:
+    """Check the validity of the URL. Currently it's only based
+    on the scheme. Whether the scheme is in allowed schemes or not
+    """
+    return get_url_scheme(url) in ALLOWED_SCHEMES
+
+
 class RuleLabels(Enum):
     PIP_NAME_MAPPING = "pip_name_mapping"
     CONDA_NAME_MAPPING = "conda_name_mapping"
     SYSTEM = "system"
 
 
-@dataclass
-class PrintableWarning:
-    package_name: str
-    message: str
-
-    def __str__(self):
-        return f"{self.package_name}: {self.message}"
-
-
 @functools.lru_cache(maxsize=None)
 def _get_rule_maps() -> Dict[str, Dict[str, str]]:
     """
@@ -122,80 +138,250 @@ def _source2parsed_pypi(name: str) -> str:
     return pypi_rules.get(name, name).replace("-", "_")
 
 
-@dataclass
-class BasePackage:
+def _get_distribution_url(dist: pkg_resources.Distribution) -> Optional[str]:
+    """
+    Fetch the distribution URL from package distribution based on pep440
+    and pep610. Different details about how this feature eventually landed
+    in `pip` and different considerations can be found at the issue page -
+    https://github.com/pypa/pip/issues/609.
+    """
+    try:
+        url_meta = json.loads(dist.get_metadata(PACKAGE_URL_METAFILE))
+    except (FileNotFoundError, ValueError, UnicodeDecodeError, KeyError):
+        return None
+    url = url_meta.get("url")
+    if not url:
+        return None
+    if "vcs_info" in url_meta:
+        commit_id = url_meta["vcs_info"].get("commit_id")
+        vcs = url_meta["vcs_info"].get("vcs")
+        return f"{vcs}+{url}@{commit_id}"
+    return url
+
+
+def _is_dist_editable(dist: pkg_resources.Distribution) -> bool:
+    """
+    Check if a Distribution is installed as editable
+    """
+    for path_item in sys.path:
+        egg_link = Path(path_item) / dist.project_name / '.egg-link'
+        if egg_link.is_file():
+            return True
+    return False
+
+
+def _get_url_n_rev(string):
+    """ If the url is in the format `url@head
+
+    Example: https://github.com/org/repo@branch
+    """
+    if "@" in string:
+        return string.rsplit("@", maxsplit=1)
+    return string, None
+
+
+def _get_valid_stem(string: str) -> Optional[str]:
+    """
+    Validate and get the stem from the given path / URL
+
+    It verifies whether the given path is valid or not and if valid
+    fetches the stem from the path and consider that as the name. This
+    doesn't work in all the cases (eg. hangar-py) but it is probably the
+    most reasonable assumption that would work in almost all the cases
+    """
+    string = string.strip()
+    scheme = get_url_scheme(string).lower()
+    if scheme == "file":
+        raise RuntimeError(f"Local paths are not portable '{string}'")
+    if scheme and scheme not in ALLOWED_SCHEMES:
+        raise RuntimeError(f"Valid URL schemes are {ALLOWED_SCHEMES}")
+
+    if is_valid_url(string):
+        url, _ = _get_url_n_rev(string)
+        parsed = urlparse(url)
+        if parsed.path:
+            path = Path(parsed.path)
+        else:
+            raise RuntimeError(f"Invalid URL '{string}'")
+
+    else:
+        path = Path(string)
+        if path.is_absolute():
+            raise RuntimeError(f"Absolute paths might not resolve '{path}'")
+        if path.exists():
+            cwd = Path.cwd()
+            absolute = path.resolve()
+            if cwd not in absolute.parents and cwd != absolute:
+                raise RuntimeError(f"Path outside of current working "
+                                   f"directory are not portable {path}")
+        else:
+            raise RuntimeError(f"Error while parsing {string}")
+    return path.resolve().stem.split(".")[0]
+
+
+def _guess_valid_name(string):
+    """
+    Heuristics to fallback when `Requirement.parse` fails. This function tries
+    to find a valid name from the given string and return. This name is
+    then used to make comparison from the names from other sources.
+
+    Evidently, pip's parsing logic is exhaustive and we wouldn't want to copy
+    the whole pip parser to our source code. The heuristics this function
+    supports are a given below. Every other string would lead to a
+    warning or an exception. More details on parsing logic, checkout pip's
+    source code or the grammar + parser specified in pip508
+
+        location: /src/pip/_internal/req/constructors.py#L291
+        commit: a089883a8
+
+        - Current directory
+        - Wheel files or sub directories in current directory
+        - Valid URLs (git, remote wheel file, remote zip file etc)
+        - Valid Requirement spec with options (pkg==0.1 -i <url>)
+    """
+    string = string.strip()
+    if string.startswith("-e "):
+        raise RuntimeError("Found editable requirements which might not be"
+                           f"portable {string}")
+    if "@ " in string:
+        # name@ path is not a valid specification in 3.7
+        name, _ = string.split("@ ", maxsplit=1)
+        if not name:
+            raise RuntimeError(f"Error while parsing '{string}'")
+        return name.strip()
+    marker_sep = '; ' if is_valid_url(string) else ";"
+    substr = string.split(marker_sep, 1)[0]
+    # If options are present in the string (pkg=0.1 -i <url>)
+    splits = re.split(r" -[\w\-]+ ", substr)
+    if len(splits) > 1:
+        try:
+            req = Requirement.parse(splits[0])
+        except Exception:  # noqa
+            raise RuntimeError(f"Error while parsing {string}")
+        name = req.project_name
+    else:
+        name = _get_valid_stem(substr)
+    return name
+
+
+class BasePackage(abc.ABC):
     """Base Package class for holding the PIP and Conda Spec.
 
     Custom Package classes must be inherited from this class to include
     custom logic for equality check and parsing
-    """
-    name: str
 
+    Note: Parameter `name` comes from the name of the package defined
+    by the developer of the package. But in cases where we cannot infer
+    that name, we fallback to other methods. For example, in the case
+    of a URL, the name is the `stem` of the `path` in the URL. If the
+    name is identified by none of the methods, we generate a uuid instead
+    """
+    def __init__(self, name: str):
+        self.name = name
+        self.parsed_name = self.name.replace("-", "_")
+
+    @abc.abstractmethod
+    def from_string(self, string):
+        raise NotImplementedError
+
+    @abc.abstractmethod
     def __eq__(self, other: BasePackage) -> bool:
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def __str__(self):
+        raise NotImplementedError
+
 
-@dataclass
 class CondaPackage(BasePackage):
     """
     Do not call the constructor / init directly. Use classmethod(s) instead
     """
-    version: Optional[str] = None
-    build: Optional[str] = None
-
-    def __post_init__(self):
+    def __init__(self, name: Optional[str], version: Optional[str],
+                 build: Optional[str]):
+        super().__init__(name)
         self.parsed_name = _conda2parsed_pypi(self.name)
+        self.version = version
+        self.build = build
 
     @classmethod
-    def from_string(cls, string):
-        try:
-            name, version, build = string.split("=")
-        except ValueError:
-            raise RuntimeError("Error while parsing the dependency spec from "
-                               "yaml file. Raise the issue with Grid support")
-        if not all((name, version, build)):
-            raise ValueError("Arguments cannot be undefined for conda spec")
+    def from_string(cls, string: str) -> CondaPackage:
+        splits = re.split(r"\W*=\W*", string)
+        if len(splits) > 3:
+            raise RuntimeError(f"ParsingError: Conda spec {string} is "
+                               f"not readable")
+        expected_elem_count = 3  # name, version and build
+        splits = splits + [None] * (expected_elem_count - len(splits))  # noqa
+        name, version, build = splits
         return cls(name=name, version=version, build=build)
 
     def __str__(self):
-        return f"{self.name}={self.version}={self.build}"
+        not_none = [
+            val for val in (self.name, self.version, self.build) if val
+        ]
+        return "=".join(not_none)
 
     def __eq__(self, other: CondaPackage):
         if other.__class__ is self.__class__:
-            return all(
-                (self.build == other.build, self.version == other.version,
-                 self.parsed_name == other.parsed_name))
+            return all((self.version == other.version,
+                        self.parsed_name == other.parsed_name))
         return NotImplemented
 
 
-@dataclass
 class PipPackage(BasePackage):
     """
     Offload all parsing, de-parsing, checks etc to `Requirement` object.
     Do not call the constructor / init directly. Use classmethod(s) instead
     """
-    def __post_init__(self):
-        # This is assigned here only for documentation purpose. The value
-        # will actually populated in `from_requirement` method
-        self.parsed_name = self.name.replace("-", "_")
-        self.requirement_obj: Optional[Requirement] = None
+    def __init__(self,
+                 name: str,
+                 req_obj: Optional[Requirement] = None,
+                 original_string: Optional[str] = None):
+        super().__init__(name)
+        self._requirement_obj = req_obj
+        self._original_string = original_string
 
     @classmethod
-    def from_string(cls, string):
-        # Not catching parsing error
-        # pkg_resources.extern.packaging.requirements.InvalidRequirement
-        # This will be raised for the requirements installed with
-        # `python setup.py develop`. However `pip install -e .` doesn't
-        # have this issue
-        req = Requirement.parse(string)
-        return cls.from_requirement(req)
+    def from_string(cls, string) -> PipPackage:
+        string = string.strip()
+        try:
+            req = Requirement.parse(string)
+        except Exception:  # noqa
+            # generic exception since different python versions have
+            # inconsistencies in raising exceptions
+            obj = None
+        else:
+            obj = cls.from_requirement(req)
+
+        if obj is None:
+            name = _guess_valid_name(string)
+            if name is None:
+                raise RuntimeError(f"Guessing name for '{string}' failed")
+            obj = cls(name=name, original_string=string)
+        return obj
 
     @classmethod
-    def from_requirement(cls, req):
-        self = cls(req.project_name)
-        self.requirement_obj = req
+    def from_requirement(cls, req: pkg_resources.Requirement):
+        self = cls(req.project_name, req_obj=req)
         return self
 
+    @classmethod
+    def from_distribution(cls, dist: pkg_resources.Distribution):
+        if _is_dist_editable(dist):
+            raise RuntimeError(f"Found editable installation which might have"
+                               f"local changes {dist.project_name}")
+        req = dist.as_requirement()
+        url = _get_distribution_url(dist)
+        # if the URL is available, we'll remove the specifier (version info)
+        # from the Requirement object since req has a bug when str(req) is
+        # invoked for a req object with both version and URL. Nevertheless,
+        # we wouldn't need the version info if url is present
+        if url:
+            req.url = url
+            req.specifier = SpecifierSet("")
+        # TODO: identify hashCmp, marker when possible
+        return cls.from_requirement(req)
+
     def _version_check(self, other: PipPackage):
         """
         Checking the versions of two instances with special constraints.
@@ -218,31 +404,56 @@ class PipPackage(BasePackage):
         Note: This check is based on the strong assumption that `other` is
         also a `PipPackage` but not a `CondaPackage`
         """
-        if len(self.requirement_obj.specifier) > 1:
+        self_has_version = bool(len(self._requirement_obj.specifier))
+        other_has_version = bool(len(other._requirement_obj.specifier))
+        if not self_has_version or not other_has_version:
+            return NotImplemented
+        if not bool(len(self._requirement_obj.specifier)):
+            return NotImplemented
+        if len(self._requirement_obj.specifier) > 1:
             # expecting other.__eq__ to solve the equality check
             return NotImplemented
-        specifier = tuple(self.requirement_obj.specifier)[0]
+        specifier = tuple(self._requirement_obj.specifier)[0]
         if specifier.operator != "==":
             # expecting other.__eq__ to solve the equality check
             return NotImplemented
         # checking at-least one specifier on the other side has
         # non-empty version
-        for sp in other.requirement_obj.specifier:
+        for sp in other._requirement_obj.specifier:
             if sp.version:
                 break
         else:
             return False
         # TODO: check for prereleases
-        return specifier.version in other.requirement_obj.specifier
+        return specifier.version in other._requirement_obj.specifier
 
     def __str__(self):
-        return str(self.requirement_obj)
+        if self._requirement_obj:
+            return str(self._requirement_obj)
+        if self._original_string:
+            return self._original_string
+        raise RuntimeError("Found corrupted package")
 
     def __eq__(self, other: PipPackage):
         if self.__class__ != other.__class__:
             return NotImplemented
-        if self.requirement_obj == other.requirement_obj:
+
+        if self.parsed_name != other.parsed_name:
+            raise NonComparableError
+        if self._requirement_obj is None or other._requirement_obj is None:
+            # Comparison without the Requirement object needs to be
+            # well thought out
+            raise NonComparableError
+
+        # Checking the equality on requirement object
+        if self._requirement_obj == other._requirement_obj:
             return True
+        # Checking the string representation since there are cases
+        # Requirement's __eq__ check is not working as expected
+        # pytorch-lightning-bolts installed from github is an example
+        if str(self._requirement_obj) == str(other._requirement_obj):
+            return True
+        # Version inclusion check
         return self._version_check(other)
 
 
@@ -286,7 +497,6 @@ class DependencyManagerBase(abc.ABC):
         raise NotImplementedError
 
     def _setup(self):
-        self._warnings: List[PrintableWarning] = []
         self._source_deps_list: List[str] = self._scan_source_deps()
 
         # Reading package manager specific methods to build information base
@@ -306,16 +516,12 @@ class DependencyManagerBase(abc.ABC):
             return True
         if set(self._source_deps_list) - set(self.file_deps):
             return True
-        for key in self._source_deps_list:
-            if key not in self.env_deps:
+        for key in set(self._source_deps_list).intersection(self.env_deps):
+            try:
+                if self.env_deps[key] != self.file_deps[key]:
+                    return True
+            except NonComparableError:
                 continue
-            if self.env_deps[key] != self.file_deps[key]:
-                return True
-
-    @property
-    def warnings(self) -> str:
-        warnings = "\n".join((str(w) for w in self._warnings))
-        return warnings
 
     @staticmethod
     def _scan_source_deps() -> List[str]:
@@ -376,19 +582,27 @@ class DependencyManagerBase(abc.ABC):
         # gave us explicitly and hence ignoring. Also, valid_names must keep
         # the order of values in requirements file
         missing = set(self._source_deps_list) - set(self.file_deps)
-        valid_names = list(self.file_deps.keys()) + sorted(list(missing))
-        for name in valid_names:
+        for name in itertools.chain(self.file_deps.keys(), sorted(missing)):
             if name in self.env_deps and name in self.file_deps:
-                if self.env_deps[name] == self.file_deps[name]:
+                try:
+                    is_same = self.env_deps[name] == self.file_deps[name]
                     # Preferring file deps in case both are equal to avoid
                     # losing other information such as markers
-                    collated.append(self.file_deps[name])
-                else:
-                    collated.append(self.env_deps[name])
+                    if is_same:
+                        qualified = self.file_deps[name]
+                    else:
+                        qualified = self.env_deps[name]
+                except NonComparableError:
+                    # Assuming both are same if not comparable
+                    qualified = self.file_deps[name]
             elif name in self.file_deps:
-                collated.append(self.file_deps[name])
+                qualified = self.file_deps[name]
             elif name in self.env_deps:
-                collated.append(self.env_deps[name])
+                qualified = self.env_deps[name]
+            else:
+                raise RuntimeError(f"Package name '{name}' not found in file "
+                                   f"or env")
+            collated.append(qualified)
         return collated
 
     def write_config(self, config: Dict):
@@ -457,13 +671,19 @@ class CondaManager(DependencyManagerBase):
         dependencies = {}
         pip_dependencies = []
         for d in specs:
+            # Only pip listing inside a yaml file must be a mapping
             if isinstance(d, dict):
                 if "pip" not in d or len(d) > 1:
                     raise RuntimeError("Could not parse the conda yaml file. "
                                        "Raise the issue with Grid support")
                 pip_dependencies = d["pip"]
+            # conda deps listing in a yaml file will be a string
             else:
-                dep = CondaPackage.from_string(d)
+                try:
+                    dep = CondaPackage.from_string(d)
+                except RuntimeError:
+                    raise RuntimeError("Could not parse the conda yaml file. "
+                                       "Raise the issue with Grid support")
                 dependencies[dep.parsed_name] = dep
         for elem in pip_dependencies:
             dep = PipPackage.from_string(elem)
@@ -524,6 +744,27 @@ class CondaManager(DependencyManagerBase):
         yaml.safe_dump(final, self.req_file.open(mode="w+"))
 
 
+def _yield_valid_lines(content):
+    """Yield valid lines from requirements.txt
+    """
+    lines = (line for line in content.splitlines())
+    for line in lines:
+        line = line.strip()
+        if not line or line.startswith("#"):
+            continue
+        # Drop comments but not # in a URL
+        if ' #' in line:
+            line = line[:line.find(' #')]
+        # line continuations
+        if line.endswith('\\'):
+            line = line[:-2].strip()
+            try:
+                line += next(lines)
+            except StopIteration:
+                return
+        yield line
+
+
 class PipManager(DependencyManagerBase):
     def _read_file_deps(self) -> Dict[str, PipPackage]:
         """
@@ -531,14 +772,8 @@ class PipManager(DependencyManagerBase):
         """
         file_deps = {}
         if self.req_file.exists():
-            # Not catching parsing error
-            # pkg_resources.extern.packaging.requirements.InvalidRequirement
-            # This will be raised for the requirements installed with
-            # `python setup.py develop`. However `pip install -e .` doesn't
-            # have this issue
-            for req in pkg_resources.parse_requirements(
-                    self.req_file.read_text()):
-                dep = PipPackage.from_requirement(req)
+            for ln in _yield_valid_lines(self.req_file.read_text()):
+                dep = PipPackage.from_string(ln)
                 file_deps[dep.parsed_name] = dep
         return file_deps
 
@@ -553,27 +788,10 @@ class PipManager(DependencyManagerBase):
         """
         out = {}
         working_set = pkg_resources.working_set
-        if working_set is None:
-            return out
-        # identifying dependencies of dependencies but not using that to
-        # to make more filtering at this phase
-        sub_reqs = {}
-        for ws in working_set:  # skipcq: PYL-E1133
-            for req in ws.requires():
-                dep = PipPackage.from_requirement(req)
-                sub_reqs[req.project_name] = dep
-            if self._is_dist_editable(ws):
-                warn = PrintableWarning(ws.project_name,
-                                        'Package is installed as "editable"')
-                self._warnings.append(warn)
-            req = ws.as_requirement()
-            url = self._get_distribution_url(ws)
-            if url:
-                req.url = self._get_distribution_url(ws)
-                req.specifier = SpecifierSet("")
-            # TODO: identify hashCmp, marker when possible
-            dep = PipPackage.from_requirement(req)
-            out[dep.parsed_name] = dep
+        if working_set:
+            for ws in working_set:  # skipcq: PYL-E1133
+                dep = PipPackage.from_distribution(ws)
+                out[dep.parsed_name] = dep
         return out
 
     def write_spec(self):
@@ -581,38 +799,5 @@ class PipManager(DependencyManagerBase):
         Write requirements to `requirements.txt`
         """
         collated = self.collate_final()
-        out = "\n".join((str(val) for val in collated))
+        out = "\n".join((str(val) for val in collated)) + "\n"
         Path("requirements.txt").write_text(out)
-
-    @staticmethod
-    def _get_distribution_url(
-            dist: pkg_resources.Distribution) -> Optional[str]:
-        """
-        Fetch the distribution URL from package distribution based on pep440
-        and pep610. Different details about how this feature eventually landed
-        in `pip` and different considerations can be found at the issue page -
-        https://github.com/pypa/pip/issues/609.
-        """
-        try:
-            url_meta = json.loads(dist.get_metadata(PACKAGE_URL_METAFILE))
-        except (FileNotFoundError, ValueError, UnicodeDecodeError):
-            return None
-        url = url_meta.get("url")
-        if not url:
-            return None
-        if "vcs_info" in url_meta:
-            commit_id = url_meta["vcs_info"].get("commit_id")
-            vcs = url_meta["vcs_info"].get("vcs")
-            return f"{vcs}+{url}@{commit_id}"
-        return url
-
-    @staticmethod
-    def _is_dist_editable(dist: pkg_resources.Distribution) -> bool:
-        """
-        Check if a Distribution is installed as editable
-        """
-        for path_item in sys.path:
-            egg_link = Path(path_item) / dist.project_name / '.egg-link'
-            if egg_link.is_file():
-                return True
-        return False
