@@ -14,18 +14,23 @@ Variables
 
     * `DEBUG`: If gridrunner should print additional information for
     debugging purposes.
-    
+
     * `SHOW_PROCESS_STATUS_DETAILS`: Global flag used to print Run submit details.
 
     * `SEGMENT_KEY`: Key to send data analytics to Segment.
 
-    * `SEGMENT_TRACKING`: If tracking analytics should be sent to Segment.
+    * `GRID_DISABLE_TRACKING`: If tracking should be disabled
+
+    * `IGNORE_WARNINGS`: if we should ignore warning prompts throughout the CLI.
+
+    * `GRID_SKIP_VERSION_CHECK`: skips version check between client and backend.
 
 """
-import os
 import logging
+import os
 
 from grid.utilities import get_graphql_url
+
 ENVIRONMENT = os.getenv('ENVIRONMENT', 'development')
 
 DEFAULT_GRID_URL = 'https://987bcab01b929eb2c07877b224215c92.grid.ai/graphql'
@@ -33,8 +38,8 @@ GRID_URL = get_graphql_url(os.getenv('GRID_URL', DEFAULT_GRID_URL))
 
 USER_ID = os.getenv('GRID_USER_ID')
 API_KEY = os.getenv('GRID_API_KEY')
-SEGMENT_KEY = os.getenv('SEGMENT_KEY')
-SEGMENT_TRACKING = bool(os.getenv('SEGMENT_TRACKING'))
+SEGMENT_KEY = os.getenv('SEGMENT_KEY', "VqTtJqOy7yCzJFhoyfNAT5IJusE8zVQR")
+GRID_DISABLE_TRACKING = os.getenv('GRID_DISABLE_TRACKING')
 
 DEBUG = bool(os.getenv('DEBUG'))
 
@@ -43,3 +48,9 @@ logger = logging.getLogger(__name__)  # pragma: no cover
 SHOW_PROCESS_STATUS_DETAILS = False
 
 IGNORE_WARNINGS = None
+
+SKIP_VERSION_CHECK = bool(os.getenv('GRID_SKIP_VERSION_CHECK'))
+
+GRID_SSH_CONFIG = os.getenv("GRID_SSH_CONFIG",
+                            default=os.path.join(os.path.expanduser("~"),
+                                                 ".ssh", "config"))
