@@ -1,3 +1,5 @@
+from array import array
+import base64
 import csv
 import glob
 import json
@@ -5,6 +7,7 @@ import os
 from pathlib import Path
 from unittest.mock import MagicMock
 from urllib.parse import urljoin
+import zlib
 
 import click
 from click.exceptions import ClickException
@@ -17,6 +20,7 @@ from tests.mock_backend import resolvers
 import yaspin
 
 import grid.client as grid
+from grid.datastore import DatastoreUploadSession
 from grid.exceptions import AuthenticationError
 from grid.exceptions import TrainError
 import grid.globals as env
@@ -33,7 +37,7 @@ def monkey_patch_observables():
     class MonkeyPatchedObservable:
         key = 'getRuns'
 
-        def __init__(self, *args, **kwargs):
+        def __init__(self, *args, **kwargs):  # skipcq: PTC-W0049
             pass
 
         def get(self, *args, **kwarrgs):
@@ -409,41 +413,33 @@ class TestGridClient:
         with pytest.raises(click.ClickException):
             G.delete_cluster(cluster_id='test-cluster')
 
-    # @staticmethod
-    # def test_upload_datastore():
-    #     """Grid().upload_datastore() makes correct API queries"""
-    #     G = grid.Grid(load_local_credentials=False)
-    #     G._init_client()
-
-    #     def monkey_patch_upload(*args, **kwargs):
-    #         return True
-
-    #     monkeypatch.setattr(grid.S3DatastoreUploader, 'upload',
-    #                        monkey_patch_upload):
-    #         result = G.upload_datastore(
-    #             source_dir='./tests/data/datastore_test/',
-    #             name='datastore-test',
-    #             version='v0',
-    #             credential_id='test-credential')
-    #         assert result
-
-    #         result = G.upload_datastore(
-    #             source_dir='./tests/data/datastore_test/',
-    #             name='datastore-test',
-    #             version='v0',
-    #             credential_id='test-credential',
-    #             staging_dir='tests/data/')
-    #         assert result
-
     @staticmethod
     def test_upload_datstore_handles_error():
         G = grid.Grid(load_local_credentials=False)
         G.client = MonkeyPatchClient()
 
         with pytest.raises(click.ClickException):
-            G.upload_datastore(source_dir='./tests/data/datastore_test/',
+            G.upload_datastore(source='./tests/data/datastore_test/',
                                name='datastore-test',
-                               credential_id='test-credential')
+                               credential_id='test-credential',
+                               compression=True)
+
+    @staticmethod
+    def test_resuming_upload_datastore():
+        G = grid.Grid(load_local_credentials=False)
+        G.client = MonkeyPatchClient()
+
+        session = DatastoreUploadSession(name="test_session",
+                                         version=1,
+                                         source_dir="data",
+                                         credential_id="cc-abcdef")
+        upload = MagicMock()
+        session.upload = upload
+
+        DatastoreUploadSession.recover_sessions = MagicMock(
+            return_value=[session])
+        G.resume_datastore_session("test_session-v1")
+        upload.assert_called_once()
 
     @staticmethod
     def test_experiment_details():
@@ -459,7 +455,8 @@ class TestGridClient:
         G._init_client()
 
         result = G.create_interactive_node(name='test-interactive-id',
-                                           config='')
+                                           config='',
+                                           description="test-description")
         assert result
 
         def monkey_path_create(*args, **kwargs):
@@ -469,7 +466,9 @@ class TestGridClient:
                             monkey_path_create)
         env.DEBUG = True
         with pytest.raises(click.ClickException):
-            G.create_interactive_node(name='test-interactive-id', config='')
+            G.create_interactive_node(name='test-interactive-id',
+                                      config='',
+                                      description='test-description')
 
     @staticmethod
     def test_create_interactive_node_handles_error():
@@ -478,7 +477,67 @@ class TestGridClient:
         G._check_user_github_token = lambda: True
 
         with pytest.raises(click.ClickException):
-            G.create_interactive_node(name='test-interactive-id', config='')
+            G.create_interactive_node(name='test-interactive-id',
+                                      config='',
+                                      description="test-desc")
+
+    @staticmethod
+    def test_pause_interactive_node(monkeypatch):
+        G = grid.Grid(load_local_credentials=False)
+        G._init_client()
+
+        result = G.pause_interactive_node(
+            interactive_node_name='test-interactive-id')
+        assert result
+
+        def monkey_path_pause(*args, **kwargs):
+            return {'success': False, 'message': ''}
+
+        monkeypatch.setattr(resolvers, 'pause_interactive_node',
+                            monkey_path_pause)
+        env.DEBUG = True
+        with pytest.raises(click.ClickException):
+            G.pause_interactive_node(
+                interactive_node_name='test-interactive-id')
+
+    @staticmethod
+    def test_pause_interactive_node_handles_error():
+        G = grid.Grid(load_local_credentials=False)
+        G.client = MonkeyPatchClient()
+        G._check_user_github_token = lambda: True
+
+        with pytest.raises(click.ClickException):
+            G.pause_interactive_node(
+                interactive_node_name='test-interactive-id')
+
+    @staticmethod
+    def test_resume_interactive_node(monkeypatch):
+        G = grid.Grid(load_local_credentials=False)
+        G._init_client()
+
+        result = G.resume_interactive_node(
+            interactive_node_name='test-interactive-id')
+        assert result
+
+        def monkey_path_pause(*args, **kwargs):
+            return {'success': False, 'message': ''}
+
+        monkeypatch.setattr(resolvers, 'resume_interactive_node',
+                            monkey_path_pause)
+        env.DEBUG = True
+        with pytest.raises(click.ClickException):
+            G.resume_interactive_node(
+                interactive_node_name='test-interactive-id')
+
+    @staticmethod
+    def test_resume_interactive_node_handles_error():
+        G = grid.Grid(load_local_credentials=False)
+        G.client = MonkeyPatchClient()
+        G._check_user_github_token = lambda: True
+
+        with pytest.raises(click.ClickException):
+            G.resume_interactive_node(
+                interactive_node_name='test-interactive-id')
 
     @staticmethod
     def test_delete_interactive_node(monkeypatch):
@@ -687,10 +746,12 @@ class TestGridClient:
                       load_local_credentials=False)
 
         G.client = MagicMock()
-        G.experiment_details = MagicMock(
-            return_value={'getExperimentDetails': {
-                'status': 'queued'
-            }})
+        G.experiment_details = MagicMock(return_value={
+            'getExperimentDetails': {
+                'status': 'queued',
+                'experimentId': 'test'
+            }
+        })
         G.experiment_logs('test')
         G.client.execute.assert_not_called()
 
@@ -726,10 +787,12 @@ class TestGridClient:
 
         G.client = MagicMock(Client, autospec=True)
         G._init_client = MagicMock()
-        G.experiment_details = MagicMock(
-            return_value={'getExperimentDetails': {
-                'status': 'running'
-            }})
+        G.experiment_details = MagicMock(return_value={
+            'getExperimentDetails': {
+                'status': 'running',
+                'experimentId': 'test'
+            }
+        })
         G.client.subscribe = MagicMock(return_value=[{
             'getLiveExperimentLogs': [{
                 "message": "foo",
@@ -739,10 +802,49 @@ class TestGridClient:
                 "timestamp": "date2"
             }]
         }])
+
         G.experiment_logs('test')
         G._init_client.assert_called_once()
         G.client.subscribe.assert_called_once()
 
+    def test_experiment_metrics_queued(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        G.client = MagicMock()
+        G.experiment_details = MagicMock(return_value={
+            'getExperimentDetails': {
+                'status': 'queued',
+                'experimentId': 'test'
+            }
+        })
+        G.experiment_metrics('test', 'train_loss')
+        G.client.execute.assert_not_called()
+
+    def test_experiment_metrics_succeeded(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        G.client = MagicMock(Client, autospec=True)
+        G.experiment_details = MagicMock(
+            return_value={'getExperimentDetails': {
+                'status': 'succeeded'
+            }})
+
+        # TODO: use when format = 'binary'
+        # steps = base64.b64encode(zlib.compress(b'00000000'))
+        # values = base64.b64encode(zlib.compress(b'00000000'))
+        G.client.execute = MagicMock(
+            return_value={
+                'getExperimentScalarLogs': {
+                    'steps': "[0, 1]",
+                    'values': "[0.1, 0.2]",
+                    'nextIndex': 10
+                }
+            })
+        G.experiment_metrics('test', 'train_loss')
+        G.client.execute.assert_called_once()
+
     @httpretty.activate
     def test_check_version_compatibility(self, capsys, monkeypatch):
         G = grid.Grid(credential_path=self.creds_path,
@@ -838,13 +940,11 @@ class TestGridClient:
             [
                 {
                     'name': 'ix1',
-                    'port': 233,
-                    'hostname': "ix1.grid.ai"
+                    'ssh_url': 'ssh://jovyan@ix1.grid.ai:233',
                 },
                 {
                     'name': 'ix2',
-                    'port': 433,
-                    'hostname': "ix2.grid.ai"
+                    'ssh_url': 'ssh://jovyan@ix2.grid.ai:433',
                 },
             ],
             '\n'.join([
@@ -860,11 +960,15 @@ class TestGridClient:
                 "    User jovyan",
                 "    Hostname ix1.grid.ai",
                 "    Port 233",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "Host ix2",
                 "    User jovyan",
                 "    Hostname ix2.grid.ai",
                 "    Port 433",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "#END",
             ])
@@ -875,13 +979,11 @@ class TestGridClient:
             [
                 {
                     'name': 'ix1',
-                    'port': 233,
-                    'hostname': "ix1.grid.ai"
+                    'ssh_url': 'ssh://jovyan@ix1.grid.ai:233',
                 },
                 {
                     'name': 'ix2',
-                    'port': 433,
-                    'hostname': "ix2.grid.ai"
+                    'ssh_url': 'ssh://jovyan@ix2.grid.ai:433',
                 },
             ],
             '\n'.join([
@@ -891,11 +993,15 @@ class TestGridClient:
                 "    User jovyan",
                 "    Hostname ix1.grid.ai",
                 "    Port 233",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "Host ix2",
                 "    User jovyan",
                 "    Hostname ix2.grid.ai",
                 "    Port 433",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "#END",
             ]),
@@ -907,11 +1013,15 @@ class TestGridClient:
                 "    User jovyan",
                 "    Hostname ix1.grid.ai",
                 "    Port 233",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "Host ix2",
                 "    User jovyan",
                 "    Hostname ix2.grid.ai",
                 "    Port 433",
+                "    ServerAliveInterval 20",
+                "    ServerAliveCountMax 3",
                 "    StrictHostKeyChecking accept-new",
                 "#END",
             ])
