@@ -1,9 +1,9 @@
 import click
 import pytest
 
-from grid.cli.grid_train import _check_description_isnt_too_long
 from grid.cli.grid_train import _check_is_valid_extension
 from grid.cli.grid_train import _check_run_name_is_valid
+from grid.utilities import check_description_isnt_too_long
 
 
 class TestTrainCallbacks:
@@ -47,6 +47,9 @@ class TestTrainCallbacks:
         _check_description_isnt_too_long() checks that description isn't
         too long
         """
+        d = 'a' * 199
+        assert d == check_description_isnt_too_long(None, None, d)
+
         d = 'a' * 201
         with pytest.raises(click.BadParameter):
-            _check_description_isnt_too_long(None, None, d)
+            check_description_isnt_too_long(None, None, d)
